@@ -87,7 +87,7 @@ static GQuark acpi_sensors_interface_device_file_error_quark(void) {
 	gchar *string;
 
 	if (quark == 0) {
-		string = g_strdup_printf("%s-device-file-error", sensor_interface[ACPI]);
+		string = g_strdup_printf("%s-device-file-error", SENSORS_INTERFACE_STRING(ACPI));
 		quark = g_quark_from_string(string);
 		g_free(string);
 	}
