@@ -74,7 +74,7 @@ static gboolean active_sensor_execute_alarm(ActiveSensor *active_sensor,
                                             NotifType notif_type) {
         int pid;
 
-        sensors_applet_notify(active_sensor, notif_type);
+        sensors_applet_notify_active_sensor(active_sensor, notif_type);
         g_debug("EXECUTING %s ALARM: %s", 
                 (notif_type == LOW_ALARM ? 
                  "LOW" : "HIGH"),
@@ -390,9 +390,9 @@ static void active_sensor_set_graph_dimensions(ActiveSensor *as,
 
         /* dimensions are really for graph frame, so need to remove
          * extra width added by graph frame - make sure not less than
-         * 0 */
-        graph_width = CLAMP(width - GRAPH_FRAME_EXTRA_WIDTH, 0, width - GRAPH_FRAME_EXTRA_WIDTH);
-        graph_height = CLAMP(height - GRAPH_FRAME_EXTRA_WIDTH, 0 , height - GRAPH_FRAME_EXTRA_WIDTH);
+         * 1 - always need atleast 1 sample */
+        graph_width = CLAMP(width - GRAPH_FRAME_EXTRA_WIDTH, 1, width - GRAPH_FRAME_EXTRA_WIDTH);
+        graph_height = CLAMP(height - GRAPH_FRAME_EXTRA_WIDTH, 1 , height - GRAPH_FRAME_EXTRA_WIDTH);
 
         g_debug("setting graph dimensions to %d x %d", graph_width, graph_height);
         num_samples = graph_width;
@@ -522,10 +522,14 @@ ActiveSensor *active_sensor_new(SensorsApplet *sensors_applet,
 
 static void active_sensor_update_sensor_value(ActiveSensor *as,
                                               gdouble sensor_value) {
-        memmove(&(as->sensor_values[1]),
-                as->sensor_values,
-                (as->num_samples - 1)*sizeof(gdouble));
 
+        /* only if have more than 1 sample stored */
+        if (as->num_samples > 1) {
+                memmove(&(as->sensor_values[1]),
+                        as->sensor_values,
+                        (as->num_samples - 1)*sizeof(gdouble));
+        }
+        
         as->sensor_values[0] = sensor_value;
 }
 
@@ -607,8 +611,13 @@ void active_sensor_update(ActiveSensor *active_sensor,
 				
 
                         if (error) {
-			  g_debug("Error updating active sensor: %s", error->message);
-				value_text = g_strdup(error->message);
+                                g_debug("Error updating active sensor: %s", error->message);
+                                sensors_applet_notify_active_sensor(active_sensor, 
+                                                                    SENSOR_INTERFACE_ERROR);
+                                
+                                /* hard code text as ERROR */
+				value_text = g_strdup(_("ERROR"));
+                                value_tooltip = g_strdup_printf("- %s", error->message);
 				g_error_free(error);
 				error = NULL;
 
