@@ -60,7 +60,7 @@ static GQuark hddtemp_sensors_interface_error_quark(void) {
 	gchar *string;
 
 	if (quark == 0) {
-		string = g_strdup_printf("%s-device-file-error", sensor_interface[HDDTEMP]);
+		string = g_strdup_printf("%s-device-file-error", SENSORS_INTERFACE_STRING(HDDTEMP));
 		quark = g_quark_from_string(string);
 		g_free(string);
 	}
