@@ -51,8 +51,9 @@
 		| grep -iv -f regexes | sort -u
 #endif
 
-#define VOLTAGE_EXPS_LENGTH 7
+#define VOLTAGE_EXPS_LENGTH 8
 static const char *volt_exps[VOLTAGE_EXPS_LENGTH] = {
+	"in",
 	"vcore",
 	"[0-9]v",
 	"^v",
@@ -227,7 +228,7 @@ static void libsensors_sensors_interface_get_sensors(SensorsApplet *sensors_appl
 						
 					gchar *url;
 							
-					type = get_sensor_type (label);
+					type = get_sensor_type (data->name);
 					visible = (type == TEMP_SENSOR ? TRUE : FALSE);
 					icon = get_sensor_icon(type);
 			
