@@ -29,16 +29,6 @@
 /* always list current version */
 static const gchar * const compatible_versions[] = {
         PACKAGE_VERSION,
-        "1.7.11",
-        "1.7.10",
-        "1.7.9",
-        "1.7.8",
-        "1.7.7",
-        "1.7.6",
-        "1.7.5",
-        "1.7.4",
-        "1.7.3",
-        "1.7.2"
 };
 
 #define NUM_COMPATIBLE_VERSIONS G_N_ELEMENTS(compatible_versions)
@@ -195,7 +185,7 @@ void sensors_applet_gconf_setup(SensorsApplet *sensors_applet) {
                         if (sensors_applet_gconf_is_compatible(old_version)) {
                                 /* previously setup and versions match so use
                                  * old values */
-                                g_debug("Trying to set up sensors from gconf data");
+                                g_debug("GConf data is compatible. Trying to set up sensors from gconf data");
                                 if (sensors_applet_gconf_setup_sensors(sensors_applet)) {
                                         g_debug("done setting up from gconf");
                                 } else {
