@@ -219,12 +219,16 @@ static void style_set_cb(GtkWidget *widget,
         GdkPixbuf *new_icon;
         gboolean enabled;
         SensorsApplet *sensors_applet;
+        DisplayMode display_mode;
 
         sensors_applet = (SensorsApplet *)data;
 
         g_debug("set-style occurred");
-        if (sensors_applet->sensors) {
 
+        display_mode = panel_applet_gconf_get_int(sensors_applet->applet,
+                                                  DISPLAY_MODE,
+                                                  NULL);
+        if (sensors_applet->sensors) {
                 for (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(sensors_applet->sensors), &interfaces_iter); not_end_of_interfaces; not_end_of_interfaces = gtk_tree_model_iter_next(GTK_TREE_MODEL(sensors_applet->sensors), &interfaces_iter)) {
                         
                         /* reset sensors sentinel */
@@ -236,6 +240,7 @@ static void style_set_cb(GtkWidget *widget,
                                                    ENABLE_COLUMN, &enabled,
                                                    ICON_TYPE_COLUMN, &icon_type,
                                                    -1);
+                                /* update icons */
                                 new_icon = sensors_applet_load_icon(icon_type);
                                 
                                 gtk_tree_store_set(sensors_applet->sensors,
@@ -243,8 +248,12 @@ static void style_set_cb(GtkWidget *widget,
                                                    ICON_PIXBUF_COLUMN, new_icon,
                                                    -1);
                                 g_object_unref(new_icon);
-                                
-                                if (enabled) {
+
+                                /* update icons only if currently being
+                                 * displayed */
+                                if (enabled &&         
+                                    (display_mode == DISPLAY_ICON || 
+                                     display_mode == DISPLAY_ICON_WITH_VALUE)) {
                                         path = gtk_tree_model_get_path(GTK_TREE_MODEL(sensors_applet->sensors), 
                                                                        &sensors_iter);
                                         sensors_applet_icon_changed(sensors_applet,
@@ -253,8 +262,10 @@ static void style_set_cb(GtkWidget *widget,
                                 }
                         }
                 }
+                /* now update layout as size may have changed */
                 sensors_applet_display_layout_changed(sensors_applet);
-        }
+            }
+            
 }
 
 static const BonoboUIVerb sensors_applet_menu_verbs[] = {
@@ -264,8 +275,69 @@ static const BonoboUIVerb sensors_applet_menu_verbs[] = {
 	BONOBO_UI_VERB_END
 };
 
+#ifdef HAVE_LIBNOTIFY
+static void notif_closed_cb(NotifyNotification *notification,
+                            SensorsApplet *sensors_applet) 
+{
+        g_assert(sensors_applet);
+        
+        sensors_applet->notification = NULL;
+}
+#endif // HAVE_LIBNOTIFY
 
-void sensors_applet_notify(ActiveSensor *active_sensor, NotifType notif_type) {
+void sensors_applet_notify(SensorsApplet *sensors_applet,
+                           NotifType notif_type) 
+{
+#ifdef HAVE_LIBNOTIFY
+        gchar *message;
+        gchar *summary;
+        GError *error = NULL;
+        g_assert(sensors_applet);
+        
+        if (!notify_is_initted()) {
+                if (!notify_init(PACKAGE)) {
+                        return;
+                }
+        }
+
+        if (sensors_applet->notification) {
+                g_debug("notification already shown, not showing another one...");
+                return;
+        }
+        
+        switch (notif_type) {
+        case GCONF_READ_ERROR:
+                summary = g_strdup_printf(_("Error restoring saved sensor configuration."));
+                message = g_strdup_printf(_("An error occurred while trying to restore the saved sensor configuration. The previous configuration has been lost and will need to be re-entered."));
+                break;
+                
+        case GCONF_WRITE_ERROR:
+                summary = g_strdup_printf(_("Error saving sensor configuration."));
+                message = g_strdup_printf(_("An error occurred while trying to save the current sensor configuration. "));
+                break;
+        }
+        
+        sensors_applet->notification = notify_notification_new(summary,
+                                                               message,
+                                                               GTK_STOCK_DIALOG_WARNING,
+                                                               sensors_applet->applet);
+        g_free(summary);
+        g_free(message);
+        
+        g_signal_connect(sensors_applet->notification,
+                         "closed",
+                         G_CALLBACK(notif_closed_cb),
+                         sensors_applet);
+        g_debug("showing notification");
+        if (!notify_notification_show(sensors_applet->notification, &error)) {
+                g_debug("Error showing notification: %s", error->message);
+                g_error_free(error);
+        } 
+#endif // HAVE_LIBNOTIFY
+}
+        
+
+void sensors_applet_notify_active_sensor(ActiveSensor *active_sensor, NotifType notif_type) {
 #ifdef HAVE_LIBNOTIFY
  
         SensorsApplet *sensors_applet;
@@ -274,7 +346,9 @@ void sensors_applet_notify(ActiveSensor *active_sensor, NotifType notif_type) {
         gchar *summary, *message;
         gint timeout_msecs;
         gchar *sensor_label;
-        gint sensor_type;
+        gchar *sensor_path;
+        SensorType sensor_type;
+        TemperatureScale temp_scale;
         GtkTreeIter iter;
         GtkTreePath *path;
         const gchar *unit_type = NULL;
@@ -283,7 +357,7 @@ void sensors_applet_notify(ActiveSensor *active_sensor, NotifType notif_type) {
         const gchar *limit_type = NULL;
         const gchar *units = NULL;
         gdouble limit_value;
-
+        
         sensors_applet = active_sensor->sensors_applet;
 
         if (!panel_applet_gconf_get_bool(sensors_applet->applet,
@@ -309,27 +383,12 @@ void sensors_applet_notify(ActiveSensor *active_sensor, NotifType notif_type) {
         }
         g_list_free(table_children);
         
-        if (active_sensor->sensor_values[0] <= active_sensor->sensor_low_value &&
-                notif_type == LOW_ALARM) {
-                relation = _("is very low");
-                limit_type = _("lower limit");
-                limit_value = active_sensor->sensor_low_value;
-        } else if (active_sensor->sensor_values[0] >= active_sensor->sensor_high_value &&
-                notif_type == HIGH_ALARM) {
-                /* assume high alarm condition */
-                relation = _("is very high");
-                limit_type = _("upper limit");
-                limit_value = active_sensor->sensor_high_value;
-        } else {
-                g_warning("Notify called when no alarm condition!");
-                return;
-        }
-
-	path = gtk_tree_row_reference_get_path(active_sensor->sensor_row);
-	if (gtk_tree_model_get_iter(GTK_TREE_MODEL(sensors_applet->sensors), 
+        path = gtk_tree_row_reference_get_path(active_sensor->sensor_row);
+        if (gtk_tree_model_get_iter(GTK_TREE_MODEL(sensors_applet->sensors), 
                                     &iter, path)) {
                 gtk_tree_model_get(GTK_TREE_MODEL(sensors_applet->sensors), &iter,
                                    LABEL_COLUMN, &sensor_label,
+                                   PATH_COLUMN, &sensor_path,
                                    SENSOR_TYPE_COLUMN, &sensor_type,
                                    -1);
         } else {
@@ -338,52 +397,89 @@ void sensors_applet_notify(ActiveSensor *active_sensor, NotifType notif_type) {
                 return;
         }
         gtk_tree_path_free(path);
-
-        switch ((SensorType)sensor_type) {
-        case TEMP_SENSOR:
-                unit_type_title = _("Temperature");
-                unit_type = _("temperature");
-                switch ((TemperatureScale)panel_applet_gconf_get_int(active_sensor->sensors_applet->applet,
-                                                   TEMPERATURE_SCALE,
-                                                   NULL)) {
-                case CELSIUS:
-                        units = UNITS_CELSIUS;
-                        break;
-                case FAHRENHEIT:
-                        units = UNITS_FAHRENHEIT;
-                        break;
-                case KELVIN:
-                        units = UNITS_KELVIN;
-                        break;
-                default:
-                        units = NULL;
+        
+        // do different stuff for different notif types
+        switch (notif_type) {
+        case LOW_ALARM: // fall thru
+        case HIGH_ALARM:
+                if (active_sensor->sensor_values[0] <= active_sensor->sensor_low_value &&
+                    notif_type == LOW_ALARM) {
+                        relation = _("is very low");
+                        limit_type = _("lower limit");
+                        limit_value = active_sensor->sensor_low_value;
+                } else if (active_sensor->sensor_values[0] >= active_sensor->sensor_high_value &&
+                           notif_type == HIGH_ALARM) {
+                        /* assume high alarm condition */
+                        relation = _("is very high");
+                        limit_type = _("upper limit");
+                        limit_value = active_sensor->sensor_high_value;
+                } else {
+                        g_warning("Alarm notify called when no alarm condition!");
+                        g_free(sensor_path);
+                        g_free(sensor_label);
+                        return;
                 }
-
+                
+                switch ((SensorType)sensor_type) {
+                case TEMP_SENSOR:
+                        unit_type_title = _("Temperature");
+                        unit_type = _("temperature");
+                        temp_scale = (TemperatureScale)panel_applet_gconf_get_int(active_sensor->sensors_applet->applet,
+                                                                                  TEMPERATURE_SCALE,
+                                                                                  NULL);
+                        
+                        switch (temp_scale) {
+                        case CELSIUS:
+                                units = UNITS_CELSIUS;
+                                break;
+                        case FAHRENHEIT:
+                                units = UNITS_FAHRENHEIT;
+                                break;
+                        case KELVIN:
+                                units = UNITS_KELVIN;
+                                break;
+                        default:
+                                units = NULL;
+                        }
+                        
+                        break;
+                case VOLTAGE_SENSOR:
+                        unit_type_title = _("Voltage");
+                        unit_type = _("voltage");
+                        units = UNITS_VOLTAGE;
                 break;
-        case VOLTAGE_SENSOR:
-                unit_type_title = _("Voltage");
-                unit_type = _("voltage");
-                units = UNITS_VOLTAGE;
+                case FAN_SENSOR:
+                        unit_type_title = _("Fan Speed");
+                        unit_type = _("fan speed");
+                        units = UNITS_RPM;
+                        break;
+                case CURRENT_SENSOR:
+                        unit_type_title = _("Current");
+                        unit_type = _("current");
+                        units = UNITS_CURRENT;
+                        break;
+                }
+                
+                timeout_msecs = (active_sensor->alarm_timeout ? MIN(DEFAULT_NOTIFY_TIMEOUT, (active_sensor->alarm_timeout * 1000)) : DEFAULT_NOTIFY_TIMEOUT);
+                
+                summary = g_strdup_printf("%s %s %s", sensor_label, unit_type_title, _("Alarm"));
+                message = g_strdup_printf("%s %s %s (%s %2.0f%s)", sensor_label, unit_type, 
+                                          relation, limit_type, limit_value, units);  
                 break;
-        case FAN_SENSOR:
-                unit_type_title = _("Fan Speed");
-                unit_type = _("fan speed");
-                units = UNITS_RPM;
-                break;
-        case CURRENT_SENSOR:
-                unit_type_title = _("Current");
-                unit_type = _("current");
-                units = UNITS_CURRENT;
+                
+        case SENSOR_INTERFACE_ERROR:
+                summary = g_strdup_printf(_("Error updating sensor %s"), sensor_label);
+                message = g_strdup_printf(_("An error occurred while trying to update the value of the sensor %s located at %s."), sensor_label, sensor_path);
+                timeout_msecs = panel_applet_gconf_get_int(active_sensor->sensors_applet->applet,
+                                                           TIMEOUT,
+                                                           NULL);
+                
                 break;
+                
+        default:
+                g_assert_not_reached();
         }
-
-        timeout_msecs = (active_sensor->alarm_timeout ? MIN(DEFAULT_NOTIFY_TIMEOUT, (active_sensor->alarm_timeout * 1000)) : DEFAULT_NOTIFY_TIMEOUT);
-
-        summary = g_strdup_printf("%s %s %s", sensor_label, unit_type_title, _("Alarm"));
-        g_debug("here");
-        message = g_strdup_printf("%s %s %s (%s %2.0f%s)", sensor_label, unit_type, 
-                                  relation, limit_type, limit_value, units);  
-
+        
         active_sensor_libnotify_notify(active_sensor,
                                        notif_type,
                                        summary,
@@ -391,6 +487,8 @@ void sensors_applet_notify(ActiveSensor *active_sensor, NotifType notif_type) {
                                        GTK_STOCK_DIALOG_WARNING,
                                        timeout_msecs,
                                        attach);
+        
+        g_free(sensor_path);
         g_free(sensor_label);
         g_free(summary);
         g_free(message);
@@ -918,6 +1016,8 @@ gboolean sensors_applet_add_sensor_full_details(SensorsApplet *sensors_applet,
 	gboolean not_end_of_interfaces = TRUE, interface_exists = FALSE;
 	gboolean not_end_of_sensors = TRUE;
 	gchar *sensor_id;
+        gchar *sensor_path;
+        SensorType sensor_type;
 	GdkPixbuf *icon;
 	GtkTreePath *tree_path;
 
@@ -973,16 +1073,22 @@ gboolean sensors_applet_add_sensor_full_details(SensorsApplet *sensors_applet,
 			/* see if have children */
 			for (not_end_of_sensors = gtk_tree_model_iter_children(GTK_TREE_MODEL(sensors_applet->sensors), &sensors_iter,  &interfaces_iter); not_end_of_sensors; not_end_of_sensors = gtk_tree_model_iter_next(GTK_TREE_MODEL(sensors_applet->sensors), &sensors_iter)) {
 				gtk_tree_model_get(GTK_TREE_MODEL(sensors_applet->sensors), &sensors_iter,
+						   PATH_COLUMN, &sensor_path,
 						   ID_COLUMN, &sensor_id,
+						   SENSOR_TYPE_COLUMN, &sensor_type,
 						   -1);
-				if (g_ascii_strcasecmp(sensor_id, id) == 0) {
+				if (g_ascii_strcasecmp(sensor_id, id) == 0 &&
+                                    g_ascii_strcasecmp(sensor_path, path) == 0  &&
+                                    sensor_type == type) {
 					/* sensor already exists so
 					 * dont add a second time */
-					g_debug("sensor with id: %s already exists in tree, not adding a second time", sensor_id);
+					g_debug("sensor with path: %s, id: %s already exists in tree, not adding a second time", sensor_path, sensor_id);
 					g_free(sensor_id);
+					g_free(sensor_path);
 					return FALSE;
 				}
 				g_free(sensor_id);
+                                g_free(sensor_path);
 			}
 							    
 			break;
@@ -1062,8 +1168,10 @@ static ActiveSensor *sensors_applet_find_active_sensor(SensorsApplet *sensors_ap
 		sensor_tree_path = gtk_tree_row_reference_get_path(((ActiveSensor *)(current_sensor->data))->sensor_row);
 
 		if (gtk_tree_path_compare(path, sensor_tree_path) == 0) {
+                        gtk_tree_path_free(sensor_tree_path);
 			return ((ActiveSensor *)(current_sensor->data));
 		}
+                gtk_tree_path_free(sensor_tree_path);
 	}
 	return NULL;
 }
